SAMPLE_RATE = 16000
CHANNELS = 1
SILENCE_THRESHOLD = 0.01
MAX_SILENCE = 1.5

# TTS Defaults
TTS_LANGUAGE = None
TTS_VOICE_NAME = None  # variant
TTS_ENGINE = None
TTS_REGION = None
TTS_RATE = None
TTS_PITCH = None
TTS_STREAM = None
